@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007 OpenedHand Ltd.
+ * Copyright (C) 2006, 2007, 2008 OpenedHand Ltd.
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
@@ -39,9 +39,9 @@
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <net/if.h>
-#include <netinet/in.h>
 #include <arpa/inet.h>
+#include <net/if.h>
+#include <ifaddrs.h>
 #include <libsoup/soup-address.h>
 
 #include "xdgmime/xdgmime.h"
@@ -50,7 +50,7 @@
 #include "gupnp-context-private.h"
 #include "gupnp-marshal.h"
 #include "gena-protocol.h"
-#include "accept-language.h"
+#include "http-headers.h"
 
 G_DEFINE_TYPE (GUPnPContext,
                gupnp_context,
@@ -72,16 +72,12 @@ enum {
         PROP_0,
         PROP_HOST_IP,
         PROP_PORT,
+        PROP_SERVER,
         PROP_SUBSCRIPTION_TIMEOUT
 };
 
 #define LOOPBACK_IP "127.0.0.1"
 
-typedef char ip_address[15+1];
-
-#define inaddrr(x) (*(struct in_addr *) &ifr->x[sizeof sa.sin_port])
-#define IFRSIZE   ((int)(size * sizeof (struct ifreq)))
-
 /**
  * Generates the default server ID.
  **/
@@ -98,77 +94,61 @@ make_server_id (void)
                                 VERSION);
 }
 
-/**
- * Looks up the IP address for interface @iface.
- *
- * http://www.linuxquestions.org/questions/showthread.php?t=425637
- **/
-gboolean
-get_ip (const char *iface, ip_address ip)
-{
-        struct ifreq *ifr;
-        struct ifreq ifrr;
-        struct sockaddr_in sa;
-        struct sockaddr ifaddr;
-        int sockfd;
-
-        if ((sockfd = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
-                return FALSE;
-
-        ifr = &ifrr;
-
-        ifrr.ifr_addr.sa_family = AF_INET;
-
-        strncpy (ifrr.ifr_name, iface, sizeof (ifrr.ifr_name));
-
-        if (ioctl (sockfd, SIOCGIFADDR, ifr) < 0)
-                return FALSE;
-
-        ifaddr = ifrr.ifr_addr;
-        strncpy (ip, inet_ntoa (inaddrr (ifr_addr.sa_data)),
-                 sizeof (ip_address));
-
-        return TRUE;
-}
-
 /**
  * Looks up the IP address of the first non-loopback network interface.
  **/
 static char *
 get_default_host_ip (void)
 {
-        struct if_nameindex *ifs;
+        struct ifaddrs *ifa_list, *ifa;
         char *ret;
-        int i;
 
-        /* List network interfaces */
-        ifs = if_nameindex ();
-        if (ifs == NULL) {
+        ret = NULL;
+
+        if (getifaddrs (&ifa_list) != 0) {
                 g_error ("Failed to retrieve list of network interfaces:\n%s\n",
                          strerror (errno));
 
                 return NULL;
         }
 
-        ret = NULL;
+        for (ifa = ifa_list; ifa != NULL; ifa = ifa->ifa_next) {
+                char ip[INET6_ADDRSTRLEN];
+                const char *p;
+                struct sockaddr_in *s4;
+                struct sockaddr_in6 *s6;
 
-        for (i = 0; ifs[i].if_index != 0 && ifs[i].if_name != NULL; i++) {
-                ip_address ip;
-
-                /* Retrieve IP address */
-                if (!get_ip (ifs[i].if_name, ip))
+                if (ifa->ifa_addr == NULL)
                         continue;
 
-                if (strcmp (ip, LOOPBACK_IP) != 0) {
-                        /* This is not a loopback interface - just what we
-                         * are looking for. */
-                        ret = g_strdup (ip);
+                if ((ifa->ifa_flags & IFF_LOOPBACK) ||
+                    !(ifa->ifa_flags & IFF_UP))
+                        continue;
 
+                p = NULL;
+
+                switch (ifa->ifa_addr->sa_family) {
+                case AF_INET:
+                        s4 = (struct sockaddr_in *) ifa->ifa_addr;
+                        p = inet_ntop (AF_INET,
+                                       &s4->sin_addr, ip, sizeof (ip));
+                        break;
+                case AF_INET6:
+                        s6 = (struct sockaddr_in6 *) ifa->ifa_addr;
+                        p = inet_ntop (AF_INET6,
+                                       &s6->sin6_addr, ip, sizeof (ip));
+                        break;
+                default:
+                        continue; /* Unknown: ignore */
+                }
+
+                if (p != NULL) {
+                        ret = g_strdup (p);
                         break;
                 }
         }
 
-        if_freenameindex (ifs);
+        freeifaddrs (ifa_list);
 
         if (!ret) {
                 /* Didn't find anything. Let's take the loopback IP. */
@@ -240,6 +220,10 @@ gupnp_context_get_property (GObject    *object,
                 g_value_set_uint (value,
                                   gupnp_context_get_port (context));
                 break;
+        case PROP_SERVER:
+                g_value_set_object (value,
+                                    gupnp_context_get_server (context));
+                break;
         case PROP_SUBSCRIPTION_TIMEOUT:
                 g_value_set_uint (value,
                                   gupnp_context_get_subscription_timeout
@@ -341,11 +325,29 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                                     G_PARAM_STATIC_NICK |
                                     G_PARAM_STATIC_BLURB));
 
+        /**
+         * GUPnPContext:server
+         *
+         * The #SoupServer HTTP server used by GUPnP.
+         **/
+        g_object_class_install_property
+                (object_class,
+                 PROP_SERVER,
+                 g_param_spec_object ("server",
+                                      "SoupServer",
+                                      "SoupServer HTTP server",
+                                      SOUP_TYPE_SERVER,
+                                      G_PARAM_READABLE |
+                                      G_PARAM_STATIC_NAME |
+                                      G_PARAM_STATIC_NICK |
+                                      G_PARAM_STATIC_BLURB));
+
         /**
          * GUPnPContext:subscription-timeout
          *
          * The preferred subscription timeout: the number of seconds after
-         * which subscriptions are renewed.
+         * which subscriptions are renewed. Set to '0' if subscriptions 
+         * are never to time out.
          **/
         g_object_class_install_property
                 (object_class,
@@ -353,8 +355,8 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                  g_param_spec_uint ("subscription-timeout",
                                     "Subscription timeout",
                                     "Subscription timeout",
-                                    GENA_MIN_TIMEOUT,
-                                    G_MAXUINT,
+                                    0,
+                                    GENA_MAX_TIMEOUT,
                                     GENA_DEFAULT_TIMEOUT,
                                     G_PARAM_READWRITE |
                                     G_PARAM_CONSTRUCT_ONLY |
@@ -380,9 +382,17 @@ default_server_handler (SoupServerContext *server_context,
         soup_message_set_status (msg, SOUP_STATUS_NOT_FOUND);
 }
 
+/**
+ * gupnp_context_get_server
+ * @context: A #GUPnPContext
+ *
+ * Return value: The #SoupServer HTTP server used by GUPnP.
+ **/
 SoupServer *
-_gupnp_context_get_server (GUPnPContext *context)
+gupnp_context_get_server (GUPnPContext *context)
 {
+        g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
+
         if (context->priv->server == NULL) {
                 context->priv->server = soup_server_new (SOUP_SERVER_PORT,
                                                          context->priv->port,
@@ -410,7 +420,7 @@ make_server_url (GUPnPContext *context)
                 context->priv->host_ip = get_default_host_ip ();
 
         /* What port are we running on? */
-        server = _gupnp_context_get_server (context);
+        server = gupnp_context_get_server (context);
         port = soup_server_get_port (server);
 
         /* Put it all together */
@@ -480,7 +490,7 @@ gupnp_context_get_port (GUPnPContext *context)
 
         g_return_val_if_fail (GUPNP_IS_CONTEXT (context), 0);
 
-        server = _gupnp_context_get_server (context);
+        server = gupnp_context_get_server (context);
         return soup_server_get_port (server);
 }
 
@@ -699,7 +709,7 @@ gupnp_context_host_path (GUPnPContext *context,
         g_return_if_fail (local_path != NULL);
         g_return_if_fail (server_path != NULL);
 
-        server = _gupnp_context_get_server (context);
+        server = gupnp_context_get_server (context);
 
         soup_server_add_handler (server, server_path, NULL,
                                  hosting_server_handler,
@@ -723,7 +733,7 @@ gupnp_context_unhost_path (GUPnPContext *context,
         g_return_if_fail (GUPNP_IS_CONTEXT (context));
         g_return_if_fail (server_path != NULL);
 
-        server = _gupnp_context_get_server (context);
+        server = gupnp_context_get_server (context);
 
         soup_server_remove_handler (server, server_path);
 }
