@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2007 OpenedHand Ltd.
+/*
+ * Copyright (C) 2007, 2008 OpenedHand Ltd.
  *
  * Author: Jorn Baayen <jorn@openedhand.com>
  *
@@ -23,7 +23,7 @@
  * SECTION:gupnp-root-device
  * @short_description: Class for root device implementations.
  *
- * #GUPnPRootDevice allows for implementing root devices. 
+ * #GUPnPRootDevice allows for implementing root devices.
  */
 
 #include <string.h>
@@ -58,7 +58,7 @@ gupnp_root_device_finalize (GObject *object)
 {
         GUPnPRootDevice *device;
         GObjectClass *object_class;
-        
+
         device = GUPNP_ROOT_DEVICE (object);
 
         g_free (device->priv->relative_location);
@@ -73,7 +73,7 @@ gupnp_root_device_dispose (GObject *object)
 {
         GUPnPRootDevice *device;
         GObjectClass *object_class;
-        
+
         device = GUPNP_ROOT_DEVICE (object);
 
         if (device->priv->group) {
@@ -146,6 +146,57 @@ gupnp_root_device_get_property (GObject    *object,
         }
 }
 
+/* Adds @udn::@resource_type to @group, together with all earlier versions
+ * of @resource_type.
+ *
+ * Note: @resource_type is modified in this function. */
+static void
+add_resource_with_earlier_versions (GSSDPResourceGroup *group,
+                                    const char         *udn,
+                                    const char         *resource_type,
+                                    const char         *location)
+{
+        char *colon, *usn;
+        int version, i;
+
+        /* Add specified version */
+        usn = g_strdup_printf ("%s::%s", (char *) udn, (char *) resource_type);
+        gssdp_resource_group_add_resource_simple (group,
+                                                  (const char *) resource_type,
+                                                  usn,
+                                                  location);
+        g_free (usn);
+
+        /* Try to parse version */
+        colon = strrchr (resource_type, ':');
+        if (G_UNLIKELY (*colon == 0))
+                return;
+
+        colon += 1;
+        if (G_UNLIKELY (*colon == 0))
+                return;
+
+        version = atoi (colon);
+        
+        /* Add earlier versions */
+        *colon = 0;
+
+        for (i = 1; i < version; i++) {
+                char *type;
+
+                type = g_strdup_printf ("%s%d", (char *) resource_type, i);
+                usn = g_strdup_printf ("%s::%s", (char *) udn, (char *) type);
+
+                gssdp_resource_group_add_resource_simple (group,
+                                                          type,
+                                                          usn,
+                                                          location);
+
+                g_free (usn);
+                g_free (type);
+        }
+}
+
 static void
 fill_resource_group (xmlNode            *element,
                      const char         *location,
@@ -153,7 +204,6 @@ fill_resource_group (xmlNode            *element,
 {
         xmlNode *child;
         xmlChar *udn, *device_type;
-        char *usn;
 
         /* Add device */
         udn = xml_util_get_child_element_content (element, "UDN");
@@ -170,18 +220,16 @@ fill_resource_group (xmlNode            *element,
 
                 return;
         }
-        
+
         gssdp_resource_group_add_resource_simple (group,
                                                   (const char *) udn,
                                                   (const char *) udn,
                                                   location);
 
-        usn = g_strdup_printf ("%s::%s", (char *) udn, (char *) device_type);
-        gssdp_resource_group_add_resource_simple (group,
-                                                  (const char *) device_type,
-                                                  (const char *) usn,
-                                                  location);
-        g_free (usn);
+        add_resource_with_earlier_versions (group,
+                                            (const char *) udn,
+                                            (const char *) device_type,
+                                            location);
 
         xmlFree (device_type);
 
@@ -201,15 +249,11 @@ fill_resource_group (xmlNode            *element,
                         if (!service_type)
                                 continue;
 
-                        usn = g_strdup_printf ("%s::%s",
-                                               (char *) udn,
-                                               (char *) service_type);
-                        gssdp_resource_group_add_resource_simple
-                                                (group,
-                                                 (const char *) service_type,
-                                                 (const char *) usn,
-                                                 location);
-                        g_free (usn);
+                        add_resource_with_earlier_versions
+                                (group,
+                                 (const char *) udn,
+                                 (const char *) service_type,
+                                 location);
 
                         xmlFree (service_type);
                 }
@@ -250,7 +294,7 @@ gupnp_root_device_constructor (GType                  type,
 
         for (i = 0; i < n_construct_params; i++) {
                 const char *par_name;
-                
+
                 par_name = construct_params[i].pspec->name;
 
                 if (strcmp (par_name, "description-doc") == 0) {
@@ -277,7 +321,7 @@ gupnp_root_device_constructor (GType                  type,
                 return NULL;
         }
 
-        element = xml_util_get_element (root_element, 
+        element = xml_util_get_element (root_element,
                                         "device",
                                         NULL);
         if (!element) {
@@ -289,7 +333,7 @@ gupnp_root_device_constructor (GType                  type,
         /* Set element */
         for (i = 0; i < n_construct_params; i++) {
                 const char *par_name;
-                
+
                 par_name = construct_params[i].pspec->name;
 
                 if (strcmp (par_name, "element") == 0) {
@@ -437,8 +481,38 @@ gupnp_root_device_new (GUPnPContext *context,
                        xmlDoc       *description_doc,
                        const char   *relative_location)
 {
+        GUPnPResourceFactory *factory;
+
+        factory = gupnp_resource_factory_get_default ();
+
+        return gupnp_root_device_new_full (context,
+                                           factory,
+                                           description_doc,
+                                           relative_location);
+}
+
+/**
+ * gupnp_root_device_new_full
+ * @context: A #GUPnPContext
+ * @factory: A #GUPnPResourceFactory
+ * @description_doc: Pointer to the device description document
+ * @relative_location: Location to use for this device, relative to the
+ * HTTP root
+ *
+ * Return value: A new #GUPnPRootDevice object.
+ **/
+GUPnPRootDevice *
+gupnp_root_device_new_full (GUPnPContext         *context,
+                            GUPnPResourceFactory *factory,
+                            xmlDoc               *description_doc,
+                            const char           *relative_location)
+{
+        g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
+        g_return_val_if_fail (GUPNP_IS_RESOURCE_FACTORY (factory), NULL);
+
         return g_object_new (GUPNP_TYPE_ROOT_DEVICE,
                              "context", context,
+                             "resource-factory", factory,
                              "root-device", NULL,
                              "description-doc", description_doc,
                              "relative-location", relative_location,
