@@ -47,31 +47,34 @@ G_BEGIN_DECLS
                 (gupnp_uuid_get_type ())
 
 GType
-gupnp_bin_base64_get_type (void) G_GNUC_CONST;
+gupnp_bin_base64_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_bin_hex_get_type (void) G_GNUC_CONST;
+gupnp_bin_hex_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_date_get_type (void) G_GNUC_CONST;
+gupnp_date_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_date_time_get_type (void) G_GNUC_CONST;
+gupnp_date_time_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_date_time_tz_get_type (void) G_GNUC_CONST;
+gupnp_date_time_tz_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_time_get_type (void) G_GNUC_CONST;
+gupnp_time_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_time_tz_get_type (void) G_GNUC_CONST;
+gupnp_time_tz_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_uri_get_type (void) G_GNUC_CONST;
+gupnp_uri_get_type (void) G_GNUC_CONST; /* string */
 
 GType
-gupnp_uuid_get_type (void) G_GNUC_CONST;
+gupnp_uuid_get_type (void) G_GNUC_CONST; /* string */
+
+#define gupnp_value_get_xml_node( value ) \
+        (xmlNode *) g_value_get_boxed ((value))
 
 #define gupnp_value_get_string( value ) \
         (const char *) g_value_get_boxed ((value))
