@@ -23,6 +23,8 @@
 #include <stdlib.h>
 
 #include "gvalue-util.h"
+#include "xml-util.h"
+#include "gupnp-types.h"
 
 gboolean
 gvalue_util_set_value_from_string (GValue     *value,
@@ -64,13 +66,13 @@ gvalue_util_set_value_from_string (GValue     *value,
                 break;
 
         case G_TYPE_INT64:
-                i = atoi (str);
+                i = atol (str);
                 g_value_set_int64 (value, (gint64) i);
 
                 break;
 
         case G_TYPE_UINT64:
-                i = atoi (str);
+                i = atol (str);
                 g_value_set_uint64 (value, (guint64) i);
 
                 break;
@@ -151,22 +153,98 @@ gvalue_util_set_value_from_string (GValue     *value,
         return TRUE;
 }
 
-char *
-gvalue_util_value_get_string (const GValue *value)
+gboolean
+gvalue_util_set_value_from_xml_node (GValue  *value,
+                                     xmlNode *node)
+{
+        xmlChar *str;
+        int ret;
+
+        str = xmlNodeGetContent (node);
+
+        ret = gvalue_util_set_value_from_string (value, (const char *) str);
+
+        xmlFree (str);
+
+        return ret;
+}
+
+gboolean
+gvalue_util_value_append_to_xml_string (const GValue *value,
+                                        GString      *str)
 {
         GValue transformed_value = {0, };
-        char *ret;
+        const char *tmp;
 
         switch (G_VALUE_TYPE (value)) {
         case G_TYPE_STRING:
-                return g_value_dup_string (value);
+                /* Escape and append */
+                tmp = g_value_get_string (value);
+                if (tmp != NULL)
+                        xml_util_add_content (str, tmp);
+
+                return TRUE;
+
+        case G_TYPE_CHAR:
+                g_string_append_c (str, g_value_get_char (value));
+
+                return TRUE;
+
+        case G_TYPE_UCHAR:
+                g_string_append_c (str, g_value_get_uchar (value));
+
+                return TRUE;
+
+        case G_TYPE_INT:
+                g_string_append_printf (str, "%d", g_value_get_int (value));
+
+                return TRUE;
+
+        case G_TYPE_UINT:
+                g_string_append_printf (str, "%u", g_value_get_uint (value));
+
+                return TRUE;
+
+        case G_TYPE_INT64:
+                g_string_append_printf (str, "%"G_GINT64_FORMAT,
+                                        g_value_get_int64 (value));
+
+                return TRUE;
+
+        case G_TYPE_UINT64:
+                g_string_append_printf (str, "%"G_GUINT64_FORMAT,
+                                        g_value_get_uint64 (value));
+
+                return TRUE;
+
+        case G_TYPE_LONG:
+                g_string_append_printf (str, "%ld", g_value_get_long (value));
+
+                return TRUE;
+
+        case G_TYPE_ULONG:
+                g_string_append_printf (str, "%lu", g_value_get_ulong (value));
+
+                return TRUE;
+
+        case G_TYPE_FLOAT:
+                g_string_append_printf (str, "%f", g_value_get_float (value));
+
+                return TRUE;
+
+        case G_TYPE_DOUBLE:
+                g_string_append_printf (str, "%g", g_value_get_double (value));
+
+                return TRUE;
 
         case G_TYPE_BOOLEAN:
                 /* We don't want to convert to "TRUE"/"FALSE" */
                 if (g_value_get_boolean (value))
-                        return g_strdup ("1");
+                        g_string_append_c (str, '1');
                 else
-                        return g_strdup ("0");
+                        g_string_append_c (str, '0');
+
+                return TRUE;
 
         default:
                 /* Try to convert */
@@ -176,17 +254,18 @@ gvalue_util_value_get_string (const GValue *value)
 
                         g_value_transform (value, &transformed_value);
 
-                        ret = g_value_dup_string (&transformed_value);
+                        tmp = g_value_get_string (&transformed_value);
+                        if (tmp != NULL)
+                                xml_util_add_content (str, tmp);
 
                         g_value_unset (&transformed_value);
 
+                        return TRUE;
                 } else {
                         g_warning ("Failed to transform value of type %s "
                                    "to a string", G_VALUE_TYPE_NAME (value));
 
-                        ret = NULL;
+                        return FALSE;
                 }
-
-                return ret;
         }
 }
